/* globals String */

/* jshint node:true, esversion: 6 */
'use strict';

var xml2js = require('xml2js');
var fs = require('fs');
var RSVP = require('rsvp');
var chalk = require('chalk');
var _findIndex = require('lodash').findIndex;
var _forOwn = require('lodash').forOwn;
var _filter = require('lodash').filter;
var _remove = require('lodash').remove;
var _pick = require('lodash').pick;
var _pullAt = require('lodash').pullAt;

var parseXML = function parseXML(xmlPath) {
  return new RSVP.Promise(function (resolve, reject) {
    var contents = fs.readFileSync(xmlPath, 'utf8');
    var parser = new xml2js.Parser();

    if (contents === '') reject('File is empty');

    parser.parseString(contents, function (err, result) {
      if (err) reject(err);
      if (result) resolve(result);
    });
  });
};

var saveXML = function saveXML(json, xmlPath) {
  var builder = new xml2js.Builder({
    renderOpts: {
      'pretty': true,
      'indent': '    ',
      'newline': '\n'
    }
  });
  var xml = builder.buildObject(json);

  // Add missing trailing newline
  fs.writeFileSync(xmlPath, xml + '\n');
};

var addNodes = function addNodes(json, opts) {
  _forOwn(opts.desiredNodes, function (nodeData, platformName) {

    //Cordova wont always have a platforms: []
    if (!json.widget.platform) json.widget.platform = [];

    //See if platform already exists
    var platformNode = void 0;
    var platformNodePos = _findIndex(json.widget.platform, { $: { name: platformName } });

    if (platformNodePos > -1) {
      //the platform existed, assign to platform Node & temp rm from JSOn
      platformNode = json.widget.platform[platformNodePos];
      _pullAt(json.widget.platform, platformNodePos);
    } else {
      platformNode = { $: { name: platformName } };
    }

    var targetNodes = platformNode[opts.keyName];
    if (targetNodes === undefined) {
      targetNodes = [];
    }

    // Icons do not have a consistent node for detection
    var idAttrs = nodeData.idAttrs;

    nodeData.sizes.forEach(function (node) {
      //If node exists, overwrite it
      var newAttrs = opts.serializeFn(platformName, opts.projectPath, node);

      _filter(targetNodes, function (item) {
        if (!item) return;

        // Compare using idAttrs because file names cannot be guaranteed to not
        // not change in future versions.
        if (_pick(item.$, idAttrs) === _pick(newAttrs, idAttrs)) {
          _remove(targetNodes, item);
        }
      });

      targetNodes.push({ $: newAttrs });
    });

    platformNode[opts.keyName] = targetNodes;
    json.widget.platform.push(platformNode);
  });

  return json;
};

/*
   Required opts:

   desiredNodes Array:
   {ios: [], android: [], blackberry: []}
   See src/platform-icon-sizes for an example

   keyName: String
   `icon` or `splash`

   serializeFn: Function
   Given a single node from desiredNodes, serialize to config.xml format
*/
module.exports = function (opts) {
  var configPath = opts.projectPath + '/config.xml';

  return new RSVP.Promise(function (resolve, reject) {
    if (!opts.projectPath || !opts.desiredNodes || !opts.keyName || !opts.serializeFn) {

      reject('Missing required opts: projectPath, desiredNodes, keyName, serializeFn');
    }

    parseXML(configPath).then(function (json) {
      json = addNodes(json, opts);
      saveXML(json, configPath);
      resolve();
    }).catch(function (err) {
      console.log(chalk.red('Error reading XML: ' + err));

      reject(new Error(err));
    });
  });
};